<?php
declare (strict_types=1);

namespace app;

use app\model\Users;
use think\App;
use think\Collection;
use think\exception\ValidateException;
use think\Validate;

/**
 * 控制器基础类
 */
abstract class BaseController
{
    /**
     * Request实例
     * @var \think\Request
     */
    protected $request;

    /**
     * 应用实例
     * @var \think\App
     */
    protected $app;

    /**
     * 是否批量验证
     * @var bool
     */
    protected $batchValidate = false;

    /**
     * 控制器中间件
     * @var array
     */
    protected $middleware = [];

    /**
     * 构造方法
     * @access public
     * @param App $app 应用对象
     */
    public function __construct(App $app)
    {
        $this->app = $app;
        $this->request = $this->app->request;

        // 控制器初始化
        $this->initialize();
    }

    // 初始化
    protected function initialize()
    {
    }

    /**
     * 验证数据
     * @access protected
     * @param array $data 数据
     * @param string|array $validate 验证器名或者验证规则数组
     * @param array $message 提示信息
     * @param bool $batch 是否批量验证
     * @return array|string|true
     * @throws ValidateException
     */
    protected function validate(array $data, $validate, array $message = [], bool $batch = false)
    {
        if (is_array($validate)) {
            $v = new Validate();
            $v->rule($validate);
        } else {
            if (strpos($validate, '.')) {
                // 支持场景
                [$validate, $scene] = explode('.', $validate);
            }
            $class = false !== strpos($validate, '\\') ? $validate : $this->app->parseClass('validate', $validate);
            $v = new $class();
            if (!empty($scene)) {
                $v->scene($scene);
            }
        }

        $v->message($message);

        // 是否批量验证
        if ($batch || $this->batchValidate) {
            $v->batch(true);
        }

        return $v->failException(true)->check($data);
    }

    /**
     * @return Users|null
     */
    protected function getUser()
    {
        return $this->request->user;
    }

    protected function getUserId()
    {
        $user = $this->getUser();
        if (empty($user)) return 0;
        return $user->uid;
    }

    protected function fail($msg, $code = 1)
    {
        return json(['code' => $code, 'message' => $msg]);
    }

    protected function success($data = null, $msg = 'success')
    {
        return json(['code' => 0, 'message' => $msg, 'data' => $data]);
    }

    protected function buildGoods(&$g){
        $picture = is_array($g) ? $g['picture']:$g->picture;
        if(substr($picture,0,4) != 'http'){
            $picture = config('app.url').'/static/'.$picture;
        }
        if(is_array($g)){
            $g['picture'] = $picture;
            $g['category'] = explode(',',$g['category']);
        }else{
            $g->picture = $picture;
            $g->category = explode(',',$g->category);
        }
        return $g;
    }
    protected function buildGoodsData($gs)
    {
        foreach($gs as &$g){
            $this->buildGoods($g);
        }
        return $gs;
    }
}
